# -*- coding: utf-8 -*-

from contuino_core import Board, Action
from .contuino_micropython import mp_helpers
import json

DEFAULT_BAUD_RATE = 115200

INFO_RUN_ONCE = "\r\nYou only need to flash micropython firmware once\r\n"
INFO_PORT_MISSING = "\r\nBoard port missing. Check device manager on Windows and lsusb on unix\r\n"


class MyBoard(Board):
    """MyBoard class"""
    baud_rate = DEFAULT_BAUD_RATE

    def __init__(self, board_port=None, baud_rate=DEFAULT_BAUD_RATE, server_address=None, server_port=None, wifi_ssid=None, wifi_password=None, username=None, name=None, message=None, actions=[]):
        self.board_port = board_port
        self.baud_rate = baud_rate
        self.server_address = server_address
        self.server_port = server_port
        self.wifi_ssid = wifi_ssid
        self.wifi_password = wifi_password
        super().__init__(username, name, message, actions)

    def flash_micropython(self):
        print(INFO_RUN_ONCE)
        mp_helpers.install_ampy()
        mp_helpers.install_esptool()
        if self.board_port == None:
            print(INFO_PORT_MISSING)
            return
        mp_helpers.flash_micropython(self.board_port, self.baud_rate)

    def init_contuino(self):
        mp_helpers.generate_main(str(self))
        mp_helpers.deploy_main(self.board_port)

    def putty_serial_prompt(self):
        mp_helpers.serial_prompt(self.board_port, self.baud_rate)


class GpioAction(Action):

    def __init__(self, pin=None, event=None, sensor=None, sensor_code=None, analog=False, value=None):
        self.pin = pin
        self.analog = analog
        super().__init__(event, value, sensor, sensor_code)



import pip
import sys
from subprocess import call
import os
import time

MP_DIR = os.path.dirname(os.path.realpath(__file__)) + os.sep

MICROPYTHON_BIN = MP_DIR + "esp8266-20180511-v1.9.4.bin"

TEMPLATES_FOLDER = MP_DIR + "templates" + os.sep
GENERATED_FOLDER = MP_DIR + "generated" + os.sep
MAIN_TEMPLATE = TEMPLATES_FOLDER + "main_template.py"


def flash_micropython(port, baud_rate):
    call([sys.executable, '-m', 'esptool', '--port', port, 'erase_flash'])
    call([sys.executable, '-m', 'esptool', '--port', str(port), '--baud', str(baud_rate),
          'write_flash', '--flash_size=detect', '0', MICROPYTHON_BIN])
    time.sleep(5)


def install_ampy():
    print("Installing adafruit-ampy")
    call([sys.executable, '-m', 'pip', 'install', 'adafruit-ampy'])


def install_esptool():
    print("Installing esptool")
    call([sys.executable, '-m', 'pip', 'install', 'esptool'])


def deploy_main(port):
    # print help
    print("Deploying main script to board on port %s" % str(port))
    call(['ampy', '--port', str(port), 'put', GENERATED_FOLDER + 'main.py'])


def serial_prompt(port, baud_rate):
    print("Running putty serial prompt")
    call(['putty', '-serial', str(port), '-sercfg', str(baud_rate)])


def generate_main(board_data):
    print("Generating main.py script for the board")
    with open(MAIN_TEMPLATE, "rt") as fin:
        with open(GENERATED_FOLDER + "main.py", "wt") as fout:
            for line in fin:
                line = line.replace("%_BOARD_DATA_%", board_data)
                fout.write(line)
